#include "OTAProxy.h"
#include "NFC.h"
#include <PubSubClient.h>
#include <ArduinoLog.h>

OTAProxy::OTAProxy(PubSubClient *mqttClient, NFC *nfc, int id)
{
    this->mqtt = mqttClient;
    this->nfc = nfc;
    this->id = id;
}

bool OTAProxy::hasActiveOTA()
{
    return activeOTA;
}

void OTAProxy::startOTA()
{
    if (!(nfc->hasCardSelected()))
    {
        return;
    }
    activeOTA = true;

    char topic[] = "fabreader/00000/startOTA";
    sprintf(topic, "fabreader/%05d/startOTA", id);

    String uid = nfc->getUID();
    mqtt->publish(topic, uid.c_str(), uid.length());

    Log.info(F("Start OTA"));
}

void OTAProxy::continueOTA(char *topic, byte *payload, unsigned int length)
{
    // Create topic strings with sufficient buffer sizes
    char topic_requestOTA[32];
    sprintf(topic_requestOTA, "fabreader/%05d/requestOTA", id);

    char topic_responseOTA[32];
    sprintf(topic_responseOTA, "fabreader/%05d/responseOTA", id);

    char topic_stopOTA[32];
    sprintf(topic_stopOTA, "fabreader/%05d/stopOTA", id);

    char topic_restartOTA[32];
    sprintf(topic_restartOTA, "fabreader/%05d/restartOTA", id);

    if (!strcmp(topic, topic_requestOTA))
    {
        Log.info("Request OTA");

        byte response[APDU_BUFFER_SIZE] = {0};

        // Maximum value for a uint8_t is 255.
        uint8_t response_len = 255; // Use 255 even if APDU_BUFFER_SIZE is 256

        // Cast length (unsigned int) to uint8_t, assuming it fits within 255 bytes.
        uint8_t sendLength = (uint8_t)length;

        Log.trace(F("Run Transceive"));
        int status = nfc->rfid->inDataExchange(payload, sendLength, response, &response_len);
        Log.trace(F("PICC_Transceive: 0x%02x"), status);

        if (status < 0)
        {
            cancelOTA();
            return;
        }

        mqtt->publish(topic_responseOTA, response, response_len);
        Log.info("Response OTA");
    }
    else if (!strcmp(topic, topic_stopOTA))
    {
        Log.info("Stop OTA");
        nfc->disconnectCard();
        activeOTA = false;
    }
    // else if (!strcmp(topic, topic_restartOTA))
    // {
    //     Log.info("Restart OTA");  // Updated if uncommented
    //     // Add your PN532-specific card deselection logic here if needed.
    // }
}

void OTAProxy::cancelOTA()
{
    char topic_cancelOTA[] = "fabreader/00000/cancelOTA";
    sprintf(topic_cancelOTA, "fabreader/%05d/cancelOTA", id);

    String uid = nfc->getUID();
    mqtt->publish(topic_cancelOTA, uid.c_str(), uid.length());

    while (!(nfc->disconnectCard()))
    {
        yield();
    }
    activeOTA = false;

    Log.info("Cancel OTA");
}